import unittest
from unittest.mock import patch, MagicMock, mock_open
import os
from datetime import datetime
from dotenv import load_dotenv
from mysql_sp_extractor.extractor import StoredProcedureExtractor

# Load environment variables
load_dotenv()

class TestStoredProcedureExtractor(unittest.TestCase):
    def setUp(self):
        self.config = {
            'host': os.getenv('DB_HOST', 'localhost'),
            'port': int(os.getenv('DB_PORT', 3306)),
            'user': os.getenv('DB_USER', 'test_user'),
            'password': os.getenv('DB_PASS', 'test_pass'),
            'database': os.getenv('DB_NAME', 'test_db')
        }
        self.extractor = StoredProcedureExtractor(**self.config)

    @patch('mysql.connector.connect')
    def test_connect_success(self, mock_connect):
        mock_connect.return_value = MagicMock()
        self.assertTrue(self.extractor.connect())
        mock_connect.assert_called_once_with(**self.config)


    @patch('mysql.connector.connect')
    def test_disconnect(self, mock_connect):
        conn_mock = MagicMock()
        conn_mock.is_connected.return_value = True
        mock_connect.return_value = conn_mock
        self.extractor.connect()
        self.extractor.disconnect()
        conn_mock.close.assert_called_once()

    @patch('os.path.exists')
    @patch('os.makedirs')
    def test_create_output_directory_new(self, mock_makedirs, mock_exists):
        mock_exists.return_value = False
        self.extractor.create_output_directory()
        mock_makedirs.assert_called_once_with(self.extractor.output_dir)

    @patch('os.path.exists')
    @patch('os.makedirs')
    def test_create_output_directory_existing(self, mock_makedirs, mock_exists):
        mock_exists.return_value = True
        self.extractor.create_output_directory()
        mock_makedirs.assert_not_called()

    @patch('mysql.connector.connect')
    def test_get_routines(self, mock_connect):
        cursor_mock = MagicMock()
        cursor_mock.fetchall.return_value = [('proc1', 'PROCEDURE'), ('proc2', 'PROCEDURE')]
        conn_mock = MagicMock()
        conn_mock.cursor.return_value = cursor_mock
        mock_connect.return_value = conn_mock
        
        self.extractor.connect()
        routines = self.extractor.get_routines()
        
        self.assertEqual(len(routines), 2)
        self.assertEqual(routines[0], ('proc1', 'PROCEDURE'))

    @patch('mysql.connector.connect')
    def test_get_routine_definition(self, mock_connect):
        cursor_mock = MagicMock()
        cursor_mock.fetchone.return_value = ('proc1', 'utf8', 'CREATE PROCEDURE proc1...')
        conn_mock = MagicMock()
        conn_mock.cursor.return_value = cursor_mock
        mock_connect.return_value = conn_mock
        
        self.extractor.connect()
        definition = self.extractor.get_routine_definition('proc1')
        
        self.assertEqual(definition, 'CREATE PROCEDURE proc1...')

    @patch('builtins.open', new_callable=mock_open)
    def test_save_routine(self, mock_file):
        result = self.extractor.save_routine('test_proc', 'CREATE PROCEDURE test_proc...')
        self.assertTrue(result)
        mock_file.assert_called_once()

    @patch('mysql.connector.connect')
    def test_extract_all(self, mock_connect):
        conn_mock = MagicMock()
        cursor_mock = MagicMock()
        
        # Mock get_routines responses
        cursor_mock.fetchall.side_effect = [
            [('proc1', 'PROCEDURE')],  # For procedures
            [('func1', 'FUNCTION')]    # For functions
        ]
        
        # Mock get_routine_definition responses
        cursor_mock.fetchone.side_effect = [
            ('proc1', 'utf8', 'CREATE PROCEDURE proc1...'),
            ('func1', 'utf8', 'CREATE FUNCTION func1...')
        ]
        
        conn_mock.cursor.return_value = cursor_mock
        conn_mock.is_connected.return_value = True
        mock_connect.return_value = conn_mock
        
        self.extractor.conn = conn_mock
        
        with patch('builtins.open', new_callable=mock_open()):
            results = self.extractor.extract_all()
            
        self.assertEqual(results['procedures'], 1)
        self.assertEqual(results['functions'], 1)

if __name__ == '__main__':
    unittest.main()


import mysql.connector
import os
from datetime import datetime
from typing import List, Tuple, Optional, Dict
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class StoredProcedureExtractor:
    """Extrae procedimientos almacenados de MemSQL/SingleStore"""
    
    def __init__(self, host: str, port: int, user: str, password: str, 
                 database: str, output_dir: str = "./stored_procedures"):
        """
        Inicializa el extractor
        
        Args:
            host: Hostname de la base de datos
            port: Puerto de conexión
            user: Usuario de la base de datos
            password: Contraseña
            database: Nombre de la base de datos
            output_dir: Directorio de salida para los archivos
        """
        self.config = {
            'host': host,
            'port': port,
            'user': user,
            'password': password,
            'database': database
        }
        self.output_dir = output_dir
        self.conn = None
    
    def connect(self) -> bool:
        """Establece conexión con la base de datos"""
        try:
            self.conn = mysql.connector.connect(**self.config)
            logger.info(f"✓ Conectado a {self.config['host']}:{self.config['port']}")
            return True
        except mysql.connector.Error as err:
            logger.error(f"✗ Error de conexión: {err}")
            return False
    
    def disconnect(self):
        """Cierra la conexión"""
        if self.conn and self.conn.is_connected():
            self.conn.close()
            logger.info("✓ Conexión cerrada")
    
    def create_output_directory(self):
        """Crea el directorio de salida si no existe"""
        if not os.path.exists(self.output_dir):
            os.makedirs(self.output_dir)
            logger.info(f"✓ Directorio creado: {self.output_dir}")
        else:
            logger.info(f"✓ Usando directorio: {self.output_dir}")
    
    def get_routines(self, routine_type: str = 'PROCEDURE') -> List[Tuple[str, str]]:
        """
        Obtiene la lista de procedimientos o funciones
        
        Args:
            routine_type: 'PROCEDURE' o 'FUNCTION'
        
        Returns:
            Lista de tuplas (nombre, tipo)
        """
        cursor = self.conn.cursor()
        query = """
            SELECT ROUTINE_NAME, ROUTINE_TYPE
            FROM INFORMATION_SCHEMA.ROUTINES
            WHERE ROUTINE_SCHEMA = %s
            AND ROUTINE_TYPE = %s
            ORDER BY ROUTINE_NAME
        """
        cursor.execute(query, (self.config['database'], routine_type))
        routines = cursor.fetchall()
        cursor.close()
        return routines
    
    def get_routine_definition(self, routine_name: str, 
                               routine_type: str = 'PROCEDURE') -> Optional[str]:
        """
        Obtiene la definición de un procedimiento o función
        
        Args:
            routine_name: Nombre del procedimiento/función
            routine_type: 'PROCEDURE' o 'FUNCTION'
        
        Returns:
            Definición SQL o None si hay error
        """
        cursor = self.conn.cursor()
        command = "SHOW CREATE PROCEDURE" if routine_type == 'PROCEDURE' else "SHOW CREATE FUNCTION"
        query = f"{command} `{routine_name}`"
        
        try:
            cursor.execute(query)
            result = cursor.fetchone()
            if result and len(result) >= 3:
                return result[2]
            return None
        except mysql.connector.Error as err:
            logger.error(f"  ✗ Error obteniendo {routine_name}: {err}")
            return None
        finally:
            cursor.close()
    
    def save_routine(self, routine_name: str, definition: str, 
                    routine_type: str = 'PROCEDURE') -> bool:
        """
        Guarda el procedimiento en un archivo
        
        Args:
            routine_name: Nombre del procedimiento
            definition: Definición SQL
            routine_type: 'PROCEDURE' o 'FUNCTION'
        
        Returns:
            True si se guardó exitosamente
        """
        suffix = "_FUNC" if routine_type == 'FUNCTION' else ""
        filename = os.path.join(self.output_dir, f"{routine_name}{suffix}.sql")
        
        # Encabezado del archivo
        header = f"""-- ============================================
-- {routine_type}: {routine_name}
-- Base de datos: {self.config['database']}
-- ============================================

"""
        
        # Contenido completo con DROP y DELIMITER
        drop_command = "DROP PROCEDURE" if routine_type == 'PROCEDURE' else "DROP FUNCTION"
        content = f"{header}{drop_command} IF EXISTS `{routine_name}`;\n\nDELIMITER $$\n\n{definition}$$\n\nDELIMITER ;\n"
        
        try:
            with open(filename, 'w', encoding='utf-8') as f:
                f.write(content)
            return True
        except Exception as e:
            logger.error(f"  ✗ Error guardando {filename}: {e}")
            return False
    
    def extract_all(self, include_functions: bool = True) -> Dict[str, int]:
        """
        Extrae todos los procedimientos y opcionalmente funciones
        
        Args:
            include_functions: Si debe extraer funciones también
        
        Returns:
            Diccionario con contadores de éxito
        """
        if not self.conn or not self.conn.is_connected():
            logger.error("No hay conexión establecida")
            return {'procedures': 0, 'functions': 0}
        
        self.create_output_directory()
        
        results = {'procedures': 0, 'functions': 0}
        
        # Extraer procedimientos
        logger.info(f"🔍 Buscando procedimientos en '{self.config['database']}'...")
        procedures = self.get_routines('PROCEDURE')
        
        if procedures:
            logger.info(f"✓ Encontrados {len(procedures)} procedimientos\n")
            logger.info("📦 Extrayendo procedimientos...")
            
            for name, routine_type in procedures:
                definition = self.get_routine_definition(name, 'PROCEDURE')
                if definition and self.save_routine(name, definition, 'PROCEDURE'):
                    logger.info(f"  ✓ {name}")
                    results['procedures'] += 1
        else:
            logger.warning("✗ No se encontraron procedimientos")
        
        # Extraer funciones
        if include_functions:
            functions = self.get_routines('FUNCTION')
            
            if functions:
                logger.info(f"\n📦 Extrayendo {len(functions)} funciones...")
                
                for name, routine_type in functions:
                    definition = self.get_routine_definition(name, 'FUNCTION')
                    if definition and self.save_routine(name, definition, 'FUNCTION'):
                        logger.info(f"  ✓ {name}")
                        results['functions'] += 1
        
        return results


import argparse
import sys
from pathlib import Path
from dotenv import load_dotenv
import os
from .extractor import StoredProcedureExtractor


def load_config_from_env(env_file: str = '.env') -> dict:
    """Carga configuración desde archivo .env"""
    if not Path(env_file).exists():
        print(f"❌ Archivo {env_file} no encontrado")
        print("\nCrea un archivo .env con el siguiente formato:")
        print("""
DB_HOST=tu_host
DB_PORT=3306
DB_USER=tu_usuario
DB_PASS=tu_password
DB_NAME=tu_base_de_datos
OUTPUT_DIR=./stored_procedures
""")
        sys.exit(1)
    
    load_dotenv(env_file)
    
    config = {
        'host': os.getenv('DB_HOST'),
        'port': int(os.getenv('DB_PORT', 3306)),
        'user': os.getenv('DB_USER'),
        'password': os.getenv('DB_PASS'),
        'database': os.getenv('DB_NAME'),
        'output_dir': os.getenv('OUTPUT_DIR', './stored_procedures')
    }
    
    # Validar campos requeridos
    required = ['host', 'user', 'password', 'database']
    missing = [k for k in required if not config[k]]
    
    if missing:
        print(f"❌ Faltan variables requeridas en {env_file}: {', '.join(missing)}")
        sys.exit(1)
    
    return config


def main():
    """Función principal del CLI"""
    parser = argparse.ArgumentParser(
        description='Extrae procedimientos almacenados de MemSQL/SingleStore',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Ejemplos:
  # Usar archivo .env por defecto
  memsql-sp-extractor
  
  # Especificar archivo .env personalizado
  memsql-sp-extractor --env .env.production
  
  # Proporcionar credenciales por línea de comandos
  memsql-sp-extractor --host localhost --user admin --database mydb
  
  # Excluir funciones
  memsql-sp-extractor --no-functions
        """
    )
    
    parser.add_argument('--env', default='.env', help='Archivo .env con configuración')
    parser.add_argument('--host', help='Hostname de la base de datos')
    parser.add_argument('--port', type=int, default=3306, help='Puerto (default: 3306)')
    parser.add_argument('--user', help='Usuario de la base de datos')
    parser.add_argument('--password', help='Contraseña')
    parser.add_argument('--database', help='Nombre de la base de datos')
    parser.add_argument('--output-dir', default='./stored_procedures', 
                       help='Directorio de salida (default: ./stored_procedures)')
    parser.add_argument('--no-functions', action='store_true',
                       help='No extraer funciones, solo procedimientos')
    parser.add_argument('--version', action='version', version='%(prog)s 0.1.0')
    
    args = parser.parse_args()
    
    # Determinar fuente de configuración
    if args.host and args.user and args.database:
        # Usar argumentos de línea de comandos
        config = {
            'host': args.host,
            'port': args.port,
            'user': args.user,
            'password': args.password or input('Password: '),
            'database': args.database,
            'output_dir': args.output_dir
        }
    else:
        # Usar archivo .env
        config = load_config_from_env(args.env)
    
    print("=" * 60)
    print("  EXTRACTOR DE PROCEDIMIENTOS ALMACENADOS")
    print("  MemSQL/SingleStore/Mysql")
    print("=" * 60)
    print()
    
    # Crear extractor y ejecutar
    extractor = StoredProcedureExtractor(**config)
    
    if not extractor.connect():
        sys.exit(1)
    
    try:
        results = extractor.extract_all(include_functions=not args.no_functions)
        
        print("\n" + "=" * 60)
        print(f"✓ Procedimientos extraídos: {results['procedures']}")
        if not args.no_functions:
            print(f"✓ Funciones extraídas: {results['functions']}")
        print(f"✓ Archivos guardados en: {os.path.abspath(config['output_dir'])}")
        print("=" * 60)
        
    finally:
        extractor.disconnect()


if __name__ == '__main__':
    main()



"""
MemSQL/Mysql Stored Procedure Extractor
Herramienta para extraer procedimientos almacenados de MemSQL/SingleStore/Mysql.
"""

__version__ = "0.1.0"
__author__ = "Ernesto Herrera Morales"
__email__ = "ernesthmdev@gmail.com"

from .extractor import StoredProcedureExtractor

__all__ = ["StoredProcedureExtractor"]


from setuptools import setup, find_packages
from pathlib import Path

# Leer el README
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding='utf-8')

setup(
    name='mysql-sp-extractor',
    version='0.1.0',
    author='Ernesto Herrera Morales',
    author_email='ernesthmdev@gmail.com',
    description='Extrae procedimientos almacenados de MemSQL/SingleStore/Mysql a archivos individuales',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/eherrera/mysql-sp-extractor',
    project_urls={
        'Bug Tracker': 'https://github.com/eherrera/mysql-sp-extractor/issues',
        'Documentation': 'https://github.com/eherrera/mysql-sp-extractor/#readme',
    },
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Database',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7',
    install_requires=[
        'mysql-connector-python>=8.0.0',
        'python-dotenv>=0.19.0',
    ],
    extras_require={
        'dev': [
            'pytest>=7.0.0',
            'pytest-cov>=3.0.0',
            'black>=22.0.0',
            'flake8>=4.0.0',
            'mypy>=0.950',
        ],
    },
    entry_points={
        'console_scripts': [
            'memsql-sp-extractor=memsql_sp_extractor.cli:main',
        ],
    },
    keywords='memsql singlestore mysql stored-procedures database backup',
)